package aa4mm.core.agent;

import aa4mm.core.coupling.GenericCouplingArtefact;
import aa4mm.core.type.SimulData;
import aa4mm.core.type.SimulTime;
import aa4mm.core.type.ValidData;
import aa4mm.core.type.ValidityInterval;
import aa4mm.deployment.netlogo.type.NetLogoTime;
/**
 * 
 * @author siebertj, V Chevrier
 * This class represents a m-agent which role is to manage a MSS block (model + simulator + software).
 * 
 * This agent is quite simple, 
 * for it only have one coupling artefact for input and one coupling artefact for output
 * 
 */

public abstract class GenericBasicMagent extends GenericMAgent {

    /**
     * The coupling-artefact used to read external input data
     */
    protected GenericCouplingArtefact readingArt;

    /**
     * The coupling-artefact used to post output data
     */
    protected GenericCouplingArtefact postingArt;

   
    public GenericBasicMagent(){
    	this.cti = new NetLogoTime(-1000);
        this.nti = new NetLogoTime(0);
    }
    
    
    /**
     * Run one step of the simulation (run step-by-step, execute one simulation event, run simulation for a given
     * simulation time interval...) And setup the cti and nti simulation time variables
     */
    protected void executeModel() {
        this.cti.setTimeFromModel(modelArtefact.getCurrentTime().getTimeValue());

        // execute the model
        modelArtefact.run();

        this.nti.setTimeFromModel(modelArtefact.getCurrentTime().getTimeValue());

    }

    /**
     * Initiate the model
     */
    public void initModel() {
        modelArtefact.init();
    }

    /**
     * @return the current simulation time
     */
    public SimulTime getCurrentTime() {
        return modelArtefact.getCurrentTime();
    }

    /**
     * Read model output data from the model artifact and post it to the
     * coupling artifact.
     */
    public void postOutputData() {
        SimulData data = modelArtefact.getOutputData();

        // assert that the data is not null (debug purposes)
        assert (data != null) : "Output Data is null";

        // create a validity interval associated with that data
        ValidityInterval v = new ValidityInterval(this.cti.getTimeValue(), this.nti.getTimeValue());

        // post the data to the output coupling-artefact
        getPostingArt().post(new ValidData(data, v));
    }
    /**
    *
    * @return The coupling-artefact used to read input data
    */
   public GenericCouplingArtefact getReadingArt() {
       // assert that the coupling-artefact exists (debug purposes)
       assert (readingArt != null) : "Reading Artefact Has not Been Set!";
	   return readingArt;
   }

   /**
    * Set the coupling-artefact used to read input data
    * @param readingArt
    */
   public void setReadingArt(GenericCouplingArtefact readingArt) {
       this.readingArt = readingArt;
   }

   /**
    *
    * @return The coupling-artefact used to post output data
    */
   public GenericCouplingArtefact getPostingArt() {
       assert (postingArt != null) : "Posting Artefact Has not Been Set!";
       return postingArt;
   }

   /**
    * set the coupling-artefact used to post output data
    * @param postingArt
    */
   public void setPostingArt(GenericCouplingArtefact postingArt) {
       this.postingArt = postingArt;
   }




}
