package aa4mm.core.agent;


import aa4mm.core.model.GenericModelArtefact;
import aa4mm.core.type.SimulData;
import aa4mm.core.type.SimulTime;


/**
 * 
 * @author siebertj, V Chevrier
 * This class represents a m-agent which role is to manage a MSS block (model + simulator + software).
 * 
 * This agent is quite simple, as it can be specialized 
 * for it only have one coupling artefact for input and one coupling artefact for output
 */
public abstract class GenericMAgent extends Thread {

	 /**
     * Simulation time values ct : lower bound of the validity interval
     * (for more detail see the coordination algorithm associated with aa4mm)
     */
    protected SimulTime cti;
	

    public SimulTime getCti() {
		return cti;
	}

	public void setCti(SimulTime cti) {
		this.cti = cti;
	}
	 /**
     * The model artifact the agent is in charge.
     */
    protected GenericModelArtefact modelArtefact;
   
	/**
     * Link this agent with the modelArtefact.
     *
     * @param modelArtefact
     */
    public void setModelArtefact(GenericModelArtefact modelArtefact) {
        this.modelArtefact = modelArtefact;
    }



	/**
     * Simulation time values ct : upper bound of the validity interval
     * (for more detail see the coordination algorithm associated with aa4mm)
     */
    protected SimulTime nti;
    public SimulTime getNti() {
		return nti;
	}
	public void setNti(SimulTime nti) {
		this.nti = nti;
	}

	/**
	 *  	Artefact management (in general)
	 */
	/**
     * Initiate the model
     */
    public abstract void initModel();
    
	 /**
     * Give the model some input data
     * @param data
     */
    /**
	 * Read model input data from the coupling artifact output port and sent it
	 * to the model artifact.
	 */
	public abstract void readInputData();

	
    public abstract void setInputData(SimulData data) ;

    /**
     * Read model output data from the model artifact and post it to the
     * coupling artifact.
     */
    public abstract void postOutputData();
    
    /**
     * Run one step of the simulation (run step-by-step, execute one simulation event, run simulation for a given
     * simulation time interval...) And setup the cti and nti simulation time variables
     */
    protected abstract void executeModel();
	
    /**
     * Call the modelArtefact finishSimulation() function
     * (useful to end logging, saved files...)
     */
    public abstract void finish();

    /**
	 * Calls the model artefact run() function
	 */
	protected void modelOneStepExecution(){
		modelArtefact.run();	
	}
   
    /**
     * The end simulation condition
     * @return true if the simulation has to be stopped
     */
    public abstract boolean endSimulation();

    

    /**
     * @return the current simulation time
     */
    public abstract SimulTime getCurrentTime() ;

	/**
	 *  method to get the initial and external parameters from the external simulators and set them to the model.
	 * @param args the initial parameters set
	 */
	public abstract void setInitialParameters(String[] args) ;

	/**
	 * This function is used in order to process the very first step of simulation. 
	 * You can use is as a regular execution steps (as in executeModel() ) or as a 
	 * specific one (if the simulator does not depend on external simulation data). 
	 * See the examples.
	 */
	public abstract void initializationProcess() ;
    
}
