package aa4mm.core.coupling;


import java.util.TreeMap;

import aa4mm.core.type.SimulData;
import aa4mm.core.type.ValidData;
import aa4mm.core.type.ValidityInterval;


/**
 * Generic Coupling Artifact for a single JVM implementation of AA4MM
 * 
 * @author siebertj
 */
public class GenericCouplingArtefact {

    /**
     * used to store the data exchanged between two managerAgents
     */
    protected TreeMap<ValidityInterval, SimulData> savedData;

    public GenericCouplingArtefact() {
        savedData = new TreeMap<ValidityInterval, SimulData>();
    }

    /**
     * Method post! Store the data into the coupling-artefact
     * @param vd
     */
    public synchronized void post(ValidData vd) {

        // assert (for debugging purposes)
        assert (vd != null) : "Don't put null validData to a coupling artefact!";
        assert (vd.getValidityInterval() != null) : "Don't put null validity interval to a coupling artefact!";
        assert (vd.getData() != null) : "Don't put null SimulData to a coupling artefact!";

        ValidityInterval v = vd.getValidityInterval();
        SimulData sd = vd.getData();
        savedData.put(v, sd);
        notifyAll();

    }

    /**
     * Method read!
     * Return the Data that are valid for the simulation time T. In this
     * implementation one coupling artefact is a single link between two models
     *
     * @param t
     * @return the data valid for the simulation time t
     */
  
    public synchronized ValidData read(double t) {

    	ValidData res;
    	while ((res=getData(t))==null)
    	{
    		try {
    			wait();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
    	}
    	return res;
       
    }
/**
 *  Method getData
 *  Return data if any which are valid 
 *  Service function 
 * @param t
 * @return the data valid for the simulation time t
 */
    private ValidData getData(double t){
    	 ValidData res = null;
         
         for (ValidityInterval validityIntervalToCheck : savedData.descendingKeySet()) {
             if (validityIntervalToCheck.isInsideValidityInterval(t)) {
                 return new ValidData(savedData.get(validityIntervalToCheck), validityIntervalToCheck);
             }
         }

         return res;
    }
    /**
     * method trans!
     * Apply a specific operation 'op' to a given Data 'sd'
     *
     * @param sd
     *            the data
     * @param op
     *            the operation to apply to sd
     * @return the result of the operation
     */
    public SimulData _applyOperation(SimulData sd, GenericOperation op) {
        return op.apply(sd);
    }
}
