package aa4mm.core.model;

import aa4mm.core.type.SimulData;
import aa4mm.core.type.SimulTime;

/**
 * This class represent a model-artefact.
 * 
 * @author Julien Siebert
 * 
 */
public abstract class GenericModelArtefact {

    /**
     * Run one simulation step or event
     */
    public abstract void run();

    /**
     *
     * @return The current simulation time
     */
    public abstract SimulTime getCurrentTime();

    /**
     * Set the model input data.
     *
     * @param data
     */
    public abstract void setInputData(SimulData data);

    /**
     * Get the model output data (to exchange with another model)
     * Ok here we only have models that give us one data
     * @return the model output data
     */
    public abstract SimulData getOutputData();

    /**
     * Create a simulator instance, load
     * the model and initiate it.
     */
    public abstract void init();

    /**
     * If a model needs initial data from the others model in order to be
     * initiated. I.e. internal model parameters are not sufficient, external
     * parameters must be set from the others coupled model.
     * @param data
     */
    public abstract void setInitialInputData(SimulData data);

    /**
     * Do something when simulation is done
     */
    public abstract void finishSimulation();

    /**
     * Set the initial parameters.
     * @param args the initial parameters
     */
    public abstract void setInitialParameters(String[] args);

    /**
     * Do the same as getOutputData but with data that serves for observation
     * @return
     */
    public abstract SimulData getObservationData();
    
}
