package aa4mm.core.type;



/**
 * 
 * @author siebertj
 *
 */
public class ValidityInterval implements Comparable<ValidityInterval> {

    private double lowerBound;
    private double upperBound;
    private static int THISISBEFORE = -1;
    private static int THISISAFTER = 1;
    private static int EQUALS = 0;

    public ValidityInterval(double lowBnd, double upBnd) {
        this.lowerBound = lowBnd;
        this.upperBound = upBnd;
        assert (lowerBound <= upperBound) : "Lower bound is greater than the upper bound in validity interval : low = " + lowBnd + ", up = " + upBnd;
    }

    /**
     * This method is used in order to retrieve the order of the validity
     * interval See composition model used by the coordination mechanism
     * introduced by Julien Siebert
     *
     * @return : Returns a negative integer, zero, or a positive integer as this
     *         object is less than, equal to, or greater than the specified
     *         object.
     */
    @Override
    public int compareTo(ValidityInterval o) {

        int compare = EQUALS;

        if (!this.equals(o)) {

            if (this.lowerBound < o.getLowerBound()) {
                compare = THISISBEFORE;
            }

            if (this.lowerBound > o.getLowerBound()) {
                compare = THISISAFTER;
            }

            if (this.lowerBound == o.getLowerBound()) {

                if (this.upperBound < o.getUpperBound()) {
                    compare = THISISBEFORE;
                } else {
                    compare = THISISAFTER;
                }

            }

        }

        return compare;

    }

    public boolean equals(ValidityInterval o) {
        return (this.lowerBound == o.lowerBound) && (this.upperBound == o.upperBound);
    }

    public double getLowerBound() {
        return lowerBound;
    }

    public void setLowerBound(double lowerBound) {
        this.lowerBound = lowerBound;
    }

    public double getUpperBound() {
        return upperBound;
    }

    public void setUpperBound(double upperBound) {
        this.upperBound = upperBound;
    }

    /**
     * A method that return true if the time value is inside the validity interval
     * See coordination algorithm from Julien Siebert
     *
     * @param timeValueToCompare
     * @return timeValueToCompare in ] lowerBound, upperBound ]
     */
    public boolean isInsideValidityInterval(double timeValueToCompare) {
        return (this.lowerBound < timeValueToCompare) && (timeValueToCompare <= this.upperBound);
    }

    @Override
    public String toString() {

        return lowerBound + "," + upperBound;
    }
}
