package examples.multilevel;


import examples.multilevel.coupling.*;
import examples.multilevel.micro.*;
import examples.multilevel.macro.*;
import util.ConsoleLogger;
import util.FilePrinter;

public class XpDownward {
	
	public final static double threshold=3.0;
	public final static double heading_threshold=20.0;
	public final static double width=71;
	public final static double height=71;
	public final static int maximumSimulationTime = 1000;
	
	public static void main(String[] args) {
		
		ConsoleLogger.getInstance().setLogging(false);
		FilePrinter.getInstance().setFileName("log.txt");
		
		ConsoleLogger.getInstance().log("Creating first agent in charge of the flocking model");
		
		// first agent manager and its model artefact (flocking model)
		AgentFlockingManager flockingManagerAgent = new AgentFlockingManager(maximumSimulationTime);
		FlockingModelArtefactDownward flockingModelInterface = new FlockingModelArtefactDownward(Parameters.pathToModels+"Flocking-Micro.nlogo");
		
		flockingManagerAgent.setModelArtefact(flockingModelInterface);
		
		
		ConsoleLogger.getInstance().log("Creating second agent in charge of the clustering model");
		
		// second agent manager and its model artefact (clustering model)
		AgentMacroManagerDownward groupsManagerAgent = new AgentMacroManagerDownward(maximumSimulationTime);
		
		MacroModelArtefact macroModelArtefact = new MacroModelArtefact(Parameters.pathToModels+"Flocking-Macro.nlogo","0");
		
		groupsManagerAgent.setModelArtefact(macroModelArtefact);
		
		ConsoleLogger.getInstance().log("Creating coupling artefacts (link between the models)");
		
		// coupling artefacts (links between models)
		UpwardArtefact flockingInterpretationArtefact = new UpwardArtefact(width, height, threshold, heading_threshold);
		DownwardArtefact artFromClusteringToFlocking = new DownwardArtefact();
		
		
		// ling agent and artefacts
		flockingManagerAgent.setPostingArt(flockingInterpretationArtefact);
		flockingManagerAgent.setReadingArt(artFromClusteringToFlocking);
		
		groupsManagerAgent.setPostingArt(artFromClusteringToFlocking);
		groupsManagerAgent.setReadingArt(flockingInterpretationArtefact);
		
		ConsoleLogger.getInstance().log("Starting the simulation of the multi-model");
		
		flockingManagerAgent.start();
		groupsManagerAgent.start();	

	}


}
