package examples.multilevel.coupling;

import examples.multilevel.coupling.optics.*;
import examples.multilevel.data_type.*;

import java.util.ArrayList;
import java.util.Vector;

import aa4mm.core.coupling.InterpretationArtefact;
import aa4mm.core.type.*;
import aa4mm.deployment.netlogo.type.*;




public class UpwardArtefact extends InterpretationArtefact{

	private ArrayList<PositionInfos> allPoints; // All the points
	private ArrayList<GroupInfos> groupFound; // Groups found
	private double threshold; // Threshold which indicate the maximum pit
	private double threshold_heading;
	

	// between

	// two elements of the same group

	public void setParameter(double myThreshold, double myThreshold_heading) {
		this.threshold = myThreshold;
		this.threshold_heading=myThreshold_heading;
	}

	public UpwardArtefact(double worldWidth, double worldHeight, double myThreshold, double myThreshold_heading) {
		this.allPoints = new ArrayList<PositionInfos>();
		this.groupFound = new ArrayList<GroupInfos>();
		this.threshold = myThreshold;
		this.threshold_heading=myThreshold_heading;
		DistanceEvaluation.getInstance().setWorldHeight(worldHeight);
		DistanceEvaluation.getInstance().setWorldWidth(worldWidth);	
	}


	/**
	 * Beginning of the algorithm Groups creation
	 */
	public SimulData interpretData(SimulData data) {
		
		allPoints=convertData(data);
		
		groupFound = new ArrayList<GroupInfos>();
		int nbGroups = 0;

		for (PositionInfos pi : allPoints) {

			/*
			 * pi is not in a group --> create one
			 */
			if (!pi.isInAGroup()) {
				nbGroups ++;
				GroupInfos gi = new GroupInfos(nbGroups);
				groupFound.add(gi);
				gi.addElement(pi);

				checkNeighborhood(gi, pi);
			}
		}
		return new GroupsFromInterpretationArtefact(groupFound);
	}

	/**
	 * Algorithm core Detailed allPoints the literature
	 * 
	 * @param gi
	 * @param neighbor
	 */
	public void checkNeighborhood(GroupInfos gi, PositionInfos neighbor) {
				CouplePointDistance[] cpd = DistanceEvaluation.getInstance().evaluate(neighbor, allPoints);

				/*
				 * The neighborhood of the current point
				 */
				ArrayList<PositionInfos> neighbors = new ArrayList<PositionInfos>();


				int i = 0;

				/*
				 * we have to check the bounds before to avoid an exception
				 */
				
				double max=neighbor.getHead()+this.threshold_heading;
				double min=neighbor.getHead()-this.threshold_heading;
				boolean bool=false;
				
				if(min<0){
					min=360+min;
					bool=true;	
				}else if(max>360){
					max=max-360;
					bool=true;		
				}
				
				while ((i < cpd.length)&& (cpd[i].getDistance() <= this.threshold)) {
					double head=cpd[i].getPi().getHead();
					if(((!bool)&&(head<=max)&&(head>=min))||(bool&&((head<=max)||(head>=min)))){
						gi.addElement(cpd[i].getPi());
						neighbors.add(cpd[i].getPi());
					}
					i++;
				}
				
				for (PositionInfos again : neighbors){
					checkNeighborhood(gi, again);
				}
	}

	
	public ArrayList<PositionInfos> convertData(SimulData data) {
		
		ArrayList<PositionInfos> array= null;
		
		if ((VectorOfNetlogoTurtles.class).equals(data.getClass())) {

			/*
			 * Add all the netlogo turtles informations to the clustering model
			 */
			
			Vector<NetlogoTurtle> v = ((VectorOfNetlogoTurtles) data)
					.getTurtleVector();
			
			array=new ArrayList<PositionInfos>();
			for (NetlogoTurtle tortuga : v) {
				PositionInfos pi = new PositionInfos(tortuga.getId(), tortuga.getxPos(), tortuga.getyPos(), tortuga.getHeading());
				array.add(pi);
			}

		} else {
			// if not (since there is no other input data the model can read)
			System.err.println("trying to send a non vector of turtle data to the interpretation artefact");
			System.exit(666);
		}
		
		return array;

	}

}
