package examples.multilevel.coupling.optics;


import java.util.ArrayList;
import java.util.Arrays;



public class DistanceEvaluation {

	private static final double defaultWorldDim = 71;

	private static DistanceEvaluation instance;

	private double worldWidth;
	private double worldHeight;
	
	public static DistanceEvaluation getInstance() {
		
		if (instance == null)
			instance = new DistanceEvaluation();
		return instance;
	}
	
	
	private DistanceEvaluation(){
		worldHeight = defaultWorldDim;
		worldWidth = defaultWorldDim;
	}
	
	

	/**
	 * Give a sorted array of CouplePointDistance objects
	 * @param pi
	 * @param in
	 * @return
	 */
	public CouplePointDistance[] evaluate(PositionInfos pi, ArrayList<PositionInfos> in) {
		
		int taille = 0;
		
		/*
		 * Size of the array 
		 */
		for (PositionInfos realSize : in){
			if (!realSize.isInAGroup()){
				taille++;
			}
		}
		
		CouplePointDistance[] array = new CouplePointDistance[taille];
		
		int i = 0;
		for (PositionInfos other : in) {
			
			if (!other.isInAGroup()) {
				array[i] = calculateDistance(pi,other);
				i++;
			}
		}
		Arrays.sort(array);
		
		return array;
	}
	
	/**
	 * Calculate distance between two points, considering the toric aspect of the grid
	 * @param pi
	 * @param other
	 * @return
	 */
	public CouplePointDistance calculateDistance(PositionInfos pi, PositionInfos other) {
		
		double distance;
		
		double xOrigin = pi.getX();
		double yOrigin = pi.getY();
		
		double xDestination = other.getX();
		double yDestination = other.getY();
		
		// What follows has been supplied by Julien Siebert
		
		
		double xClas = Math.abs(xDestination - xOrigin);
        double yClas = Math.abs(yDestination - yOrigin);
        
        double xRes = xClas;
        double yRes = yClas;
		
		/*
         * if the classical X distance between origin and dest is smaller than
         * the half of the environment then no need to cross the environment
         * borders
         */
        if (xClas > worldWidth / 2) {   
                // calculate the X component crossing the borders (torus style)
                xRes = worldWidth - xClas;              
        }

        // the same for y component
        if (yClas > worldHeight / 2) {
                yRes = worldHeight - yClas;
        }
        
        distance = Math.sqrt(xRes*xRes+yRes*yRes);
		
		return new CouplePointDistance(other, distance);
	}

	public double getWorldWidth() {
		return worldWidth;
	}

	public void setWorldWidth(double worldWidth) {
		this.worldWidth = worldWidth;
	}

	public double getWorldHeight() {
		return worldHeight;
	}

	public void setWorldHeight(double worldHeight) {
		this.worldHeight = worldHeight;
	}
}
