package examples.multilevel.coupling.optics;


import java.util.ArrayList;


public class GroupInfos {

	private int identifier;
	private ArrayList<PositionInfos> elements;
	private int size;
	private double color;
	private int worldWidth;
	private int worldHeigh;
	
	public GroupInfos(int groupId) {
		identifier = groupId;
		elements = new ArrayList<PositionInfos>();
		size = 0;
		color=0;
		worldWidth=71;
		worldHeigh=71;
	}

	public int getIdentifier() {
		return identifier;
	}

	public void setIdentifier(int identifier) {
		this.identifier = identifier;
	}

	public ArrayList<PositionInfos> getElements() {
		return elements;
	}

	public void setElements(ArrayList<PositionInfos> elements) {
		this.elements = elements;
	}

	public int getSize() {
		return size;
	}

	public void setSize(int size) {
		this.size = size;
	}

	public void addElement(PositionInfos pi) {
		elements.add(pi);
		pi.setGroup(identifier);
		size++;
	}
	
	public String toString() {
		
		String s = "";
		
		s+= "Le groupe n°"+identifier+" possède les éléments ("+size+") suivants :\n";
		for (PositionInfos pi : elements)
			s += pi.toString()+"\n";
		return s;
	}

	public double getColor(){
		return color;
	}
	
	public void setColor(double c){
		color=c;
	}
	
	public PositionInfos calculCenter(){
		double x=elements.get(0).getX();
		double y=elements.get(0).getY();
		int number=1;
	
		
		for(int i=1;i<elements.size();i++){
			PositionInfos pi=elements.get(i);
			double piX=pi.getX();
			double piY=pi.getY();
			if(Math.abs(x-piX)>worldWidth/2){
				if(piX>=0){
					piX=piX-worldWidth;
				}else{
					piX=piX+worldWidth;
				}
			}
			if(Math.abs(y-piY)>worldHeigh/2){
				if(piY>=0){
					piY=piY-worldHeigh;
				}else{
					piY=piY+worldHeigh;
				}
			}
			x=(x*number+piX)/(number+1);
			y=(y*number+piY)/(number+1);
			number++;
		}
		
		double head=0;
		double moy_head[]=new double[2];
		double nb_head[]=new double[2];
		
		for(PositionInfos pi : elements){
			if((pi.getHead()>=0)&&(pi.getHead()<180)){
				moy_head[0]+=pi.getHead();
				nb_head[0]++;
			}else{
				moy_head[1]+=pi.getHead();
				nb_head[1]++;
			}	
		}
		
		if(nb_head[0]==0){
			moy_head[1]=moy_head[1]/nb_head[1];
			head=moy_head[1];
		}else if(nb_head[1]==0){
			moy_head[0]=moy_head[0]/nb_head[0];
			head=moy_head[0];
		}else{
			moy_head[0]=moy_head[0]/nb_head[0];
			moy_head[1]=moy_head[1]/nb_head[1];
			if(moy_head[1]-moy_head[0]>180){
				moy_head[1]=moy_head[1]-360;
				head=((nb_head[0]*moy_head[0])+(moy_head[1]*nb_head[1]))/(nb_head[0]+nb_head[1]);
				if(head<0){
					head=360-head;
				}
			}else{
				head=((moy_head[0]*nb_head[0])+(moy_head[1]*nb_head[1]))/(nb_head[0]+nb_head[1]);
			}
		}
		
		
		PositionInfos center=new PositionInfos(this.getIdentifier(),x,y,head,color);
		return center;
	}
	
	public double getWidth(PositionInfos center){
		double max=0.0;
		for(PositionInfos pi : elements){
			DistanceEvaluation distance=DistanceEvaluation.getInstance();
			distance.setWorldHeight(70);
			distance.setWorldWidth(70);
			double dist=distance.calculateDistance(center, pi).getDistance();
			if(dist>max){
				max=dist;
			}
		}
		return max;
	}
}
