package examples.multilevel.macro;

import examples.multilevel.coupling.optics.DistanceEvaluation;
import examples.multilevel.coupling.optics.GroupInfos;
import examples.multilevel.coupling.optics.PositionInfos;
import examples.multilevel.data_type.GroupsFromInterpretationArtefact;
import examples.multilevel.data_type.GroupsFromMacroAgent;
import examples.multilevel.data_type.PositionsFromMacroAgent;

import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.TreeMap;
import java.util.Vector;






import util.ConsoleLogger;
import util.FilePrinter;
import aa4mm.core.agent.GenericBasicMagent;
import aa4mm.core.type.SimulData;
import aa4mm.core.type.ValidData;
import aa4mm.core.type.ValidityInterval;
import aa4mm.deployment.netlogo.type.NetLogoTime;
import aa4mm.deployment.netlogo.type.NetlogoTurtle;
import aa4mm.deployment.netlogo.type.VectorOfNetlogoTurtles;


public class AgentMacroManagerDownward extends GenericBasicMagent{
	

	
	private int maximumSimulationTime;

	
	
	private int nextcolor;
	private int begin_color;
	private ArrayList<Double> removed;
	private int nb_group_id;
	private ArrayList<GroupInfos> data;

	public AgentMacroManagerDownward(int maxSimTime) {
		this.cti = new NetLogoTime(-Double.MAX_VALUE);
		this.nti = new NetLogoTime(0);
		this.maximumSimulationTime = maxSimTime;
		
		data=new ArrayList<GroupInfos>();
		begin_color=0;
		nextcolor=4;
		removed=new ArrayList<Double>();
		nb_group_id=0;
		
	}

	@Override
	public synchronized void start() {
		ConsoleLogger.getInstance().log("Flocking Model Init Start");
		initModel();
		ConsoleLogger.getInstance().log("Flocking Model Init Done!");

		/*
		 * Initialisation process
		 */
		// send output data to other models

		super.start();
	}

	/**
	 * TODO how to wait when one artefact does not have the proper input data
	 * yet ?
	 */
	@Override
	public void run() {

		ConsoleLogger.getInstance().log("Agent Flocking Starts the simulation");

		try {
			FileWriter file_writer=new FileWriter("error.txt");

			while (!endSimulation()) {
		
				// read from all the coupling artefacts (ok, here we only have one
				// coupling artefact and we only have one data to read
				double simTime = getCurrentTime().getTimeValue();

					ValidData inputVd = getReadingArt().read(simTime);
	
					// if the coupling artefact has given me an input data
					if (inputVd != null) {
						GroupsFromInterpretationArtefact inputData = (GroupsFromInterpretationArtefact) inputVd.getData();
							
						ConsoleLogger.getInstance().log("Agent Flocking Set input data : "+ inputVd.toString());
						
						// set input data to the model
						compareData(inputData);
						
						setInputData(new GroupsFromMacroAgent(data,removed));
						
	
						ConsoleLogger.getInstance().log("Agent Flocking Execute model");
						
						executeModel();

	
						ConsoleLogger.getInstance().log("Agent Flocking Post Output Data");
	
						// send output data to other models
						postOutputData();

					} else {
						System.out.println("Run in macro agent downward: data is null");
						
					}
	
			}
			file_writer.close();
			((MacroModelArtefact)modelArtefact).saveData();
			
			
			System.exit(0);
		
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		FilePrinter.getInstance().close();
	}

	@Override
	public boolean endSimulation() {
		return (getCurrentTime()).getTimeValue() >= maximumSimulationTime;
	}
	
	public double calculError(GroupsFromMacroAgent macro){
		ArrayList<GroupInfos> groups=macro.getAgentIdGroupMap();
		Vector<NetlogoTurtle> turtles=((MacroModelArtefact)modelArtefact).getModelData().getTurtleVector();
		
		double error=0;
		int nb_error=0;
		
		for(GroupInfos g:groups){
			for(NetlogoTurtle turtle:turtles){
				if(turtle.getId()==g.getIdentifier()){
					PositionInfos p1=new PositionInfos(0,turtle.getxPos(),turtle.getyPos(),0);
					PositionInfos p2=g.calculCenter();
					double dist=DistanceEvaluation.getInstance().calculateDistance(p1,p2).getDistance();
					error+=dist;
					nb_error++;
				}
			}
		}
		error=error/nb_error;
		error=error*100.0/2;
		return error;
	}
	
    public int findMax(int i, int[][] tab){
    	int max=0;
    	for(int j=0;j<tab[i].length;j++){
    		if(tab[i][j]>tab[i][max]){
    			max=j;
    		}
    	}
    	if(tab[i][max]<=1)max=-1;
    	return max;
    }
    
    // Compare two list, Find Groups and update modele
    public void compareData(SimulData comp){
    	removed.clear();
    	ArrayList<GroupInfos> result = ((GroupsFromInterpretationArtefact)comp).getAgentIdGroupMap();
    	ArrayList<GroupInfos> modele = data;
    	if((modele!=null)&&(result!=null)){
	    	int tab[][]=new int[modele.size()][result.size()];
	    	int i=0;
	    	int j=0;
	    	
	    	// Fill the match array
	    	for(GroupInfos gi_modele : modele){
	    		for(PositionInfos pi_modele : gi_modele.getElements()){
	    			for(GroupInfos gi_result : result){
	    				for(PositionInfos pi_result: gi_result.getElements()){
	    					if(pi_result.getId()==pi_modele.getId()){
	    						tab[i][j]++;
	    					}
	    				}
	    				j++;
	    			}
	    			j=0;
	    		}
	    		i++;
	    	}
	    	
	    	int [] max_tab=new int[modele.size()];
	    	
	    	// Find the original group in the new group array (in which new group the original has the greatest match)
	    	for(i=0;i<tab.length;i++){
	    		max_tab[i]=findMax(i,tab);
	    	}
	    	
	    	// Check if 2 original group have match the same new group
	    	for(i=0;i<max_tab.length;i++){
	    		// check if the original group have a match
	    		boolean check = false;
	    		while(!check){
	    			check=true;
	    			if(max_tab[i]!=-1){
	    				// check if another original group have the same match
	    				for(j=0;j<max_tab.length;j++){
	    					if((max_tab[i]==max_tab[j])&&(max_tab[i]!=-1)&&(i!=j)){
	    						// check which original group as the greatest match with the new one
	    						if(tab[i][max_tab[i]]<tab[j][max_tab[j]]){
	    							check=false;
	    							tab[i][max_tab[i]]=0;
	    							max_tab[i]=findMax(i,tab);
	    						}else{
	    							tab[j][max_tab[j]]=0;
	    							max_tab[j]=findMax(j,tab);
	    						}
	    					}
	    				}
	    			}
	    		}
	    		
	    		//match group
	    		if(max_tab[i]!=-1){
	    			modele.get(i).setElements(result.get(max_tab[i]).getElements());
	    		}
	    	}
	    	
	    	// remove group
	    	int nb_remove=0;
	    	for(i=0;i<max_tab.length;i++){
	    		if(max_tab[i]==-1){
	    			removed.add(new Double(modele.get(i-nb_remove).getIdentifier()));
	    			modele.remove(i-nb_remove);
	    			nb_remove++;
	    		}
	    	}
	    	
	    	// check if they are new groups
	    	for(i=0;i<result.size();i++){
	    		if(result.get(i).getElements().size()>1){
		    		boolean match=false;
		    		// check if a new group is not matched
		    		for(j=0;j<max_tab.length;j++){
		    			if(max_tab[j]==i){
		    				match=true;
		    			}
		    		}
		    		if(!match){
		    			result.get(i).setIdentifier(nb_group_id);
		    			nb_group_id++;
		    			modele.add(result.get(i));
		    		}
	    		}
	    	}
    	}
    	setColor();
    }
    
    private int nextBeginColor(){
    	int[] begin={4,6,8,5,7};
    	begin_color++;
    	if(begin_color==begin.length){
    		begin_color=0;
    	}
    	return begin[begin_color];
    }
    
    private int nextColor(){
    	int color=nextcolor;
    	if(nextcolor-130>=0){
    		color=nextBeginColor();
    	}else{
    		color+=10;
    	}
    	nextcolor=color;
    	return color;
    }
	
	public void setColor(){
		for (GroupInfos groupInfos : data) {
				// find a color for each groups (see
				// http://ccl.northwestern.edu/netlogo/docs/programming.html#colors)
			double color=groupInfos.getColor();
			if(color<=0){
				color=nextcolor;
				this.nextColor();
				groupInfos.setColor(color);
			}
		}
	}
	
	public void postOutputData() {
	        SimulData data = modelArtefact.getOutputData();

	        // assert that the data is not null (debug purposes)
	        assert (data != null) : "Output Data is null";

	        // create a validity interval associated with that data
	        ValidityInterval v = new ValidityInterval(this.cti.getTimeValue(), this.nti.getTimeValue());

	        // post the data to the output coupling-artefact
	        
	        getPostingArt().post(new ValidData(prepareData(((VectorOfNetlogoTurtles)data)), v));
	 }
	
	public SimulData prepareData(VectorOfNetlogoTurtles turtles) {
		//return getNetLogoTurtlesFromModel();
		
		TreeMap<PositionInfos, double[]> pos_list=new TreeMap<PositionInfos, double[]>();
		
		for(NetlogoTurtle turtle : turtles.getTurtleVector()){
			for(GroupInfos group : data){
				if(turtle.getId()==group.getIdentifier()){
					PositionInfos center=group.calculCenter();
					double x_var=turtle.getxPos()-center.getX();
					if(x_var>35.5){
						x_var=x_var-71;
					}else if(x_var<-35.5){
						x_var=x_var+71;
					}
					double y_var=turtle.getyPos()-center.getY();
					if(y_var>35.5){
						y_var=y_var-71;
					}else if(y_var<-35.5){
						y_var=y_var+71;
					}
					double head_var=turtle.getHeading()-center.getHead();
					if(head_var>180){
						head_var=head_var-360;
					}else if(head_var<-180){
						head_var=head_var+360;
					}
					
					for(PositionInfos pos : group.getElements()){
						double tab[]={x_var,y_var,head_var};
						pos_list.put(new PositionInfos(pos.getId(),pos.getX(),pos.getY(),turtle.getHeading(),group.getColor()),tab);
					}
				}
			}
		}
		return new PositionsFromMacroAgent(pos_list);
	}

	@Override
	public void readInputData() {
		// TODO Auto-generated method stub
		
	}

	
	@Override
	public void finish() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void setInitialParameters(String[] args) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void initializationProcess() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void setInputData(SimulData data) {
        modelArtefact.setInputData(data);
    }
	
}

