package examples.multilevel.macro;

import examples.multilevel.coupling.optics.GroupInfos;
import examples.multilevel.coupling.optics.PositionInfos;
import examples.multilevel.data_type.GroupsFromMacroAgent;

import java.util.ArrayList;

import org.nlogo.agent.AgentSet;
import org.nlogo.api.CompilerException;

import aa4mm.core.type.SimulData;
import aa4mm.deployment.netlogo.model.NetlogoModelArtefact;
import aa4mm.deployment.netlogo.type.VectorOfNetlogoTurtles;

public class MacroModelArtefact extends NetlogoModelArtefact{

	String modelPath;
	
	String outpath;

	/**
	 * A constructor.
	 * 
	 * @param width
	 * @param height
	 */
	
	public MacroModelArtefact(String modelPath, String out) {
		super(modelPath, "Flocking model Macro ");
		
		outpath=out;
	}

	
	
	@Override
	public SimulData getOutputData() {
		return getNetLogoTurtlesFromModel();
	}
	
	public VectorOfNetlogoTurtles getModelData(){
		return (VectorOfNetlogoTurtles)getNetLogoTurtlesFromModel();
	}

	

	@Override
	public void run() {
		try {
			// run the go procedure (here our policy is to run only one-by-one
			comp.command("go");
		} catch (CompilerException e) {
			e.printStackTrace();
		}
	}



	

	@Override
	public void setInitialParameters(String[] args) {
		// 

	}

	

	@Override
	public void finishSimulation() {
		//
	}

	

	@Override
	public void setInitialInputData(SimulData data) {
		// 

	}

	@Override
	public void setInputData(SimulData data) {

		if ((GroupsFromMacroAgent.class).equals(data.getClass())) {
			
			GroupsFromMacroAgent groups=((GroupsFromMacroAgent)data);
			
			ArrayList<GroupInfos> allGroups= (ArrayList<GroupInfos>)(((GroupsFromMacroAgent)data).getAgentIdGroupMap()).clone();
			if (allGroups != null){ 
				if(!allGroups.isEmpty()) {
					for (GroupInfos group : allGroups) {
						changePosition(group);
					}
				}
			}
			
			ArrayList<Double> removed=groups.getRemoved();
			if(!removed.isEmpty()){
				for(Double d:removed){
					this.removeGroup(d);
				}
			}
			
			

		} else {
			// if not (since there is no other input data the model can read)
			// exit.
			System.err
					.println("trying to send a non GroupsFromInterpretationArtefact data to the groups model");
			System.exit(666);
		}

	}
	
	private void removeGroup(Double id) {
		String cmd="ask turtles with [ id = "+id+" ][die]";
		try {
			comp.command(cmd);
		} catch (CompilerException e) {
			e.printStackTrace();
		}
	}


	// -----------------------------------------------------

	/**
	 * Change the position of a group or create it
	 */
	private void changePosition(GroupInfos group) {
		// netlogo command
		PositionInfos center=group.calculCenter();
		
		try {
			String cmd = "turtles with [id = "+group.getIdentifier()+" ]";
			
			AgentSet list=(AgentSet)(comp.report(cmd));
			if(list.count()==1){
				cmd="ask turtles with [ id = "+group.getIdentifier()+" ] [ set width "+group.getWidth(center)+" set col "+group.getColor()
				+" setxy "+center.getX()+" "+center.getY()+" set heading "+center.getHead()+" ]";
			}else if(list.count()==0){
				cmd = "crt 1 [ set id "+group.getIdentifier()+" set width "+group.getWidth(center)+" set col "+group.getColor()
				+" setxy "+center.getX()+" "+center.getY()+" set heading "+center.getHead()+" set size 1.5 ]";
			}
			comp.command(cmd);
		} catch (CompilerException e) {
			e.printStackTrace();
		}
	}


	public void saveData() {
		try {
			comp.command("export-plot \"Population\" \"plots_StarExpe"+outpath+".csv\" ");
		} catch (CompilerException e) {
			e.printStackTrace();
		}
	}

	// -----------------------------------------------------

  

}

