package examples.multilevel.micro;

import examples.multilevel.coupling.optics.PositionInfos;
import examples.multilevel.data_type.PositionsFromDownwardArtefact;

import java.util.ArrayList;

import org.nlogo.api.CompilerException;

import aa4mm.core.type.SimulData;
import aa4mm.core.type.SimulTime;
import aa4mm.deployment.netlogo.model.NetlogoModelArtefact;
import aa4mm.deployment.netlogo.type.NetLogoTime;
import aa4mm.deployment.netlogo.type.NetlogoTurtle;




public class FlockingModelArtefactDownward extends NetlogoModelArtefact {



	/**
	 * A constructor.
	 * 
	 * @param width
	 * @param height
	 */
	public FlockingModelArtefactDownward(String myModelPath) {
		super(myModelPath, "Flocking model Micro ");
	}

	

	@Override
	public SimulData getOutputData() {
		return getNetLogoTurtlesFromModel();
	}

	

	@Override
	public void run() {
		try {
			// run the go procedure (here our policy is to run only one-by-one
			comp.command("go");
		} catch (CompilerException e) {
			e.printStackTrace();
		}
	}



	

	@Override
	public void setInitialParameters(String[] args) {
		// //TODO future : common to micro artefact

	}

	@Override
	public SimulTime getCurrentTime() {
		SimulTime currentTime = new NetLogoTime();
		try {
			currentTime.setTimeFromModel(new Double((Double)(comp.report("ticks"))+0.1));
		} catch (CompilerException e) {
			e.printStackTrace();
			System.exit(666);
		}
		return currentTime;
	}

	@Override
	public void finishSimulation() {
	}

	

	@Override
	public void setInitialInputData(SimulData data) {
	}

	@Override
	public void setInputData(SimulData data) {
		if ((PositionsFromDownwardArtefact.class).equals(data.getClass())) {
			

			ArrayList<PositionInfos> allGroups = ((PositionsFromDownwardArtefact)data).getPos();
			
			try {
				comp.command("ask turtles [ set colo 139.9 ]");
				comp.command("ask turtles [ set in-a-group 0 ]");
				
				if (allGroups != null){
					if(!allGroups.isEmpty()) {
						for (PositionInfos pos : allGroups) {
							changeColor((int)(pos.getId()), (int)(pos.getColor()));

							changePosition(pos);
							
						}
					}
				}
	
				comp.command("repaint");
				
			} catch (CompilerException e) {
				e.printStackTrace();
			}
			
		} else {
			// if not (since there is no other input data the model can read)
			// exit.
			System.err.println("trying to send a non GroupsFromInterpretationArtefact data to the flocking model");
			System.exit(666);
		}

	}
	
	// -----------------------------------------------------

	/**
	 * Change color of a givent turtle
	 */
	private void changeColor(int tortugaId, int colorNb) {
		// netlogo command

		String cmd = "ask " + NetlogoTurtle.getBreed() + " " + tortugaId
				+ " [ set colo " + colorNb + " set color "+colorNb+" ]";

		try {
			comp.command(cmd);

		} catch (CompilerException e) {
			e.printStackTrace();
		}
	}
	
	private void changePosition(PositionInfos pos){
		String cmd = "ask "+NetlogoTurtle.getBreed()+ " " +pos.getId()+" [ setxy "+pos.getX()+" "+pos.getY()+" set heading "+pos.getHead()+" set in-a-group 1 ]";
		try {
			comp.command(cmd);
			
		} catch (CompilerException e) {
			e.printStackTrace();
		} catch (Exception e){
			e.printStackTrace();
		}
	}


	
    @Override
    public SimulData getObservationData() {
        throw new UnsupportedOperationException("Not supported yet.");
    }


    

	// -----------------------------------------------------


}
